% Exam1: Least-squares
% Created by Quoc Tran-Dinh, STOR892, UNC-Chapel Hill.

% Generate data
n = 350; 
p = 1000;
s = 100;

A     = randn(n, p)/sqrt(n);
idxs  = randperm(p, s);
x_org = zeros(p, 1);
x_org(idxs) = randn(s, 1);
sig   = 1e-3;
b     = A*x_org + sig*randn(n, 1);

% Compute the Lipschitz constant and form the function.
lambda  = 1e-2;
fx      = @(x)( 0.5*norm(A*x - b, 2)^2 );
grad_fx = @(x) A'*(A*x - b);
L       = norm(A'*A, 2);
gx      = @(x)( lambda*norm(x(:), 1) );
proxg   = @(x, rho)( sign(x).*max(abs(x) - lambda*rho, 0) );

% Other parameters.
x0       = zeros(p, 1);
maxiters = 1000;
tol      = 1e-15;

% Call the solver.
[xopt1, output1]  = non_acc_prox_grad_alg(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
[xopt2, output2]  = non_acc_ls_prox_grad_alg(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
[xopt3, output3]  = acc_prox_grad_alg(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
[xopt4, output4]  = acc_ls_prox_grad_alg(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
[xopt5, output5]  = acc_prox_grad_alg2(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
[xopt6, output6]  = acc_ls_prox_grad_alg2(fx, grad_fx, gx, proxg, L, x0, maxiters, tol);
fmin = min([output1.fx; output2.fx; output3.fx; output4.fx; output5.fx; output6.fx]);

% Print the results.
fprintf('*************** Final result ****************\n');
fprintf('The objective value: %3.6f\n', output1.fx);
fprintf('The error: %3.6f\n', output1.rel_err);
fprintf('The number of iterations: %3d\n', output1.iter);
fprintf('The computational time: %3.2f\n', output1.time);

%% Plot the results
figure(1);
subplot(1,2,1);
title('The decrease of the objective value');
semilogy((output1.hist.fx - fmin)/max(abs(fmin), 1), 'b--', 'LineWidth', 2); hold on;
semilogy((output2.hist.fx - fmin)/max(abs(fmin), 1), 'r-.', 'LineWidth', 2); hold on;
semilogy((output3.hist.fx - fmin)/max(abs(fmin), 1), 'g', 'LineWidth', 2); hold on;
semilogy((output4.hist.fx - fmin)/max(abs(fmin), 1), 'k:', 'LineWidth', 2); hold on;
semilogy((output5.hist.fx - fmin)/max(abs(fmin), 1), 'm', 'LineWidth', 2); hold on;
semilogy((output6.hist.fx - fmin)/max(abs(fmin), 1), 'c', 'LineWidth', 2); hold on;
hold off;
xlabel('Number of iterations', 'FontSize', 12);
ylabel('The relative objective residual', 'FontSize', 12);
h1 = legend('Gradient', 'LS-Gradient', 'Fast gradient', 'LS fast gradient', 'RS fast gradient', 'LS-RS fast gradient');
set(h1, 'FontSize', 12);

subplot(1,2,2);
title('The relative error');
semilogy(output1.hist.rel_err, 'b--', 'LineWidth', 2); hold on;
semilogy(output2.hist.rel_err, 'r-.', 'LineWidth', 2); hold on;
semilogy(output3.hist.rel_err, 'g', 'LineWidth', 2); hold on;
semilogy(output4.hist.rel_err, 'k:', 'LineWidth', 2); hold on;
semilogy(output5.hist.rel_err, 'm', 'LineWidth', 2); hold on;
semilogy(output6.hist.rel_err, 'c', 'LineWidth', 2); hold on;
hold off;
xlabel('Number of iterations', 'FontSize', 12);
ylabel('The relative error', 'FontSize', 12);
h2 = legend('Gradient', 'LS-Gradient', 'Fast gradient', 'LS fast gradient', 'RS fast gradient', 'LS-RS fast gradient');
set(h2, 'FontSize', 12);

%% Plot solution.
figure(2);
subplot(1,2,1);
stairs(xopt6, '--sb' ); hold on; 
stairs(x_org, '-.ro')
subplot(1,2,2);
stairs(xopt5, '--sb' ); hold on; 
stairs(x_org, '-.ro')

