%% Example 2: Logistic regression.

% Input data.
dirData       = '';
mat_file_name = 'w4a.mat';
fprintf('+ Read the data from file: %s ...\n', mat_file_name);
mdata = load([dirData, mat_file_name]);
yhat  = full(mdata.yhat);
What  = full(mdata.What);
if isempty(yhat) || isempty(What)
    error('Can not read the data from the file!');
end
bias = 0;
bhat = bias*yhat;
p = size(What,2);
N = size(What,1);
fprintf(' + We are solving problem: %s (p = %d, N = %d)\n', ...
           mat_file_name, p, N);

% Add a strong convexity term.
lambda  = 1e-3;
mu      = 0.01;
fx      = @(x)( logistic_fx_eval(x, What, bhat, N, yhat, 1) + 0.5*mu*norm(x(:), 2)^2 );      
grad_fx = @(x)( logistic_fx_eval(x, What, bhat, N, yhat, 0) + mu*x(:) );
gx      = @(x)( lambda*norm(x(:), 1) );
proxg   = @(x, rho)( sign(x).*max(abs(x) - lambda*rho, 0) );
     
% Call the solver.
fprintf(' + CALL THE SOLVER (The normal proximal-gradient method) ...\n');
Lf       = 0.25*norm(What'*What, 2)/N + mu;

% Other parameters.
x0       = zeros(p, 1);
maxiters = 200;
tol      = 1e-15;

% Call the solver.
[xopt1, output1]  = non_acc_prox_grad_alg(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
[xopt2, output2]  = non_acc_ls_prox_grad_alg(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
[xopt3, output3]  = acc_prox_grad_alg(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
[xopt4, output4]  = acc_ls_prox_grad_alg(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
[xopt5, output5]  = acc_prox_grad_alg2(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
[xopt6, output6]  = acc_ls_prox_grad_alg2(fx, grad_fx, gx, proxg, Lf, x0, maxiters, tol);
fmin = min([output1.fx; output2.fx; output3.fx; output4.fx; output5.fx; output6.fx]);

% Print the results.
fprintf('*************** Final result ****************\n');
fprintf('The objective value: %3.6f\n', output1.fx);
fprintf('The error: %3.6f\n', output1.rel_err);
fprintf('The number of iterations: %3d\n', output1.iter);
fprintf('The computational time: %3.2f\n', output1.time);

%% Plot the results
figure(1);
subplot(1,2,1);
title('The decrease of the objective value');
semilogy((output1.hist.fx - fmin)/max(abs(fmin), 1), 'b--', 'LineWidth', 2); hold on;
semilogy((output2.hist.fx - fmin)/max(abs(fmin), 1), 'r-.', 'LineWidth', 2); hold on;
semilogy((output3.hist.fx - fmin)/max(abs(fmin), 1), 'g', 'LineWidth', 2); hold on;
semilogy((output4.hist.fx - fmin)/max(abs(fmin), 1), 'k:', 'LineWidth', 2); hold on;
semilogy((output5.hist.fx - fmin)/max(abs(fmin), 1), 'm', 'LineWidth', 2); hold on;
semilogy((output6.hist.fx - fmin)/max(abs(fmin), 1), 'c', 'LineWidth', 2); hold on;
hold off;
xlabel('Number of iterations', 'FontSize', 12);
ylabel('The relative objective residual', 'FontSize', 12);
h1 = legend('Gradient', 'LS-Gradient', 'Fast gradient', 'LS fast gradient', 'RS fast gradient', 'LS-RS fast gradient');
set(h1, 'FontSize', 12);

subplot(1,2,2);
title('The relative error');
semilogy(output1.hist.rel_err, 'b--', 'LineWidth', 2); hold on;
semilogy(output2.hist.rel_err, 'r-.', 'LineWidth', 2); hold on;
semilogy(output3.hist.rel_err, 'g', 'LineWidth', 2); hold on;
semilogy(output4.hist.rel_err, 'k:', 'LineWidth', 2); hold on;
semilogy(output5.hist.rel_err, 'm', 'LineWidth', 2); hold on;
semilogy(output6.hist.rel_err, 'c', 'LineWidth', 2); hold on;
hold off;
xlabel('Number of iterations', 'FontSize', 12);
ylabel('The relative error', 'FontSize', 12);
h2 = legend('Gradient', 'LS-Gradient', 'Fast gradient', 'LS fast gradient', 'RS fast gradient', 'LS-RS fast gradient');
set(h2, 'FontSize', 12);